package cd.casic.ci.common.pipeline.service.utils;



import cd.casic.framework.commons.util.watcher.Watcher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 日志工具类
 */
public final class LogUtils {

    private static final Logger LOG = LoggerFactory.getLogger(LogUtils.class);
    private static final int DEFAULT_MAX_LENGTH = 16384;

    private LogUtils() {
        // 私有构造函数，防止实例化
    }

    /**
     * 记录耗时信息
     * @param message 日志消息
     * @param startTime 开始时间
     * @param warnThreshold 警告阈值，默认1000ms
     * @param errorThreshold 错误阈值，默认5000ms
     */
    public static void costTime(String message, long startTime, long warnThreshold, long errorThreshold) {
        long cost = System.currentTimeMillis() - startTime;
        if (cost < warnThreshold) {
            LOG.info("{} cost {} ms", message, cost);
        } else if (cost < errorThreshold) {
            LOG.warn("{} cost {} ms", message, cost);
        } else {
            LOG.error("{} cost {} ms", message, cost);
        }
    }

    /**
     * 记录耗时信息（使用默认阈值）
     * @param message 日志消息
     * @param startTime 开始时间
     */
    public static void costTime(String message, long startTime) {
        costTime(message, startTime, 1000L, 5000L);
    }

    /**
     * 计算watcher的耗时并打印日志
     * @param watcher 监控器
     * @param warnThreshold 警告阈值
     * @param errorThreshold 错误阈值
     */
    public static void printCostTimeWE(Watcher watcher, long warnThreshold, long errorThreshold) {
        watcher.stop();
        long cost = System.currentTimeMillis() - watcher.getCreateTime();
        if (cost >= warnThreshold) {
            if (cost > errorThreshold) {
                LOG.error("{} cost {} ms", watcher, cost);
            } else {
                LOG.warn("{} cost {} ms", watcher, cost);
            }
        }
    }

    /**
     * 计算watcher的耗时并打印日志（使用默认阈值）
     * @param watcher 监控器
     */
    public static void printCostTimeWE(Watcher watcher) {
        printCostTimeWE(watcher, 1000L, 5000L);
    }

    /**
     * 获取有限长度的日志内容（使用默认最大长度16K）
     * @param logStr 原始日志内容
     * @return 截取后的日志
     */
    public static String getLogWithLengthLimit(String logStr) {
        return getLogWithLengthLimit(logStr, DEFAULT_MAX_LENGTH);
    }

    /**
     * 获取有限长度的日志内容
     * @param logStr 原始日志内容
     * @param maxLength 最大长度，若小于0则不生效，返回原始日志
     * @return 截取后的日志
     */
    public static String getLogWithLengthLimit(String logStr, int maxLength) {
        if (logStr == null) {
            return null;
        }
        if (maxLength < 0) {
            return logStr;
        }
        return logStr.length() > maxLength ? logStr.substring(0, maxLength) : logStr;
    }
}