package cd.casic.ci.process.api.process.utils;

import cd.casic.ci.common.pipeline.constant.HttpStatus;
import cd.casic.ci.common.pipeline.container.Container;
import cd.casic.ci.common.pipeline.container.NormalContainer;
import cd.casic.ci.common.pipeline.container.Stage;
import cd.casic.ci.common.pipeline.container.VMBuildContainer;
import cd.casic.ci.common.pipeline.enums.DependOnType;
import cd.casic.ci.common.pipeline.option.JobControlOption;
import cd.casic.ci.common.pipeline.pojo.DependOnConfig;
import cd.casic.ci.common.pipeline.utils.EnvUtils;
import cd.casic.ci.process.api.process.constant.ProcessMessageCode;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.regex.Pattern;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.process.api.process.utils
 * @Project：ops-pro
 * @name：DependOnUtils
 * @Date：2025/03/27 14:39
 * @Filename：DependOnUtils
 * @description：Todo
 */
@Slf4j
public class DependOnUtils {

    private static final Pattern regex = Pattern.compile("[,;]");

    public static void checkRepeatedJobId(Stage stage) {
        Set<String> jobIdSet = new HashSet<>();
        for (Container c : stage.getContainers()) {
            String jobId = c.getJobId();
            if (jobId == null || jobId.isEmpty()) {
                continue;
            }
            if (jobIdSet.contains(jobId)) {
                String jobName = getContainerName(stage, c, jobId);
                log.error(
                        HttpStatus.CONFLICT.getReasonPhrase(),
                        ProcessMessageCode.ERROR_PIPELINE_JOBID_EXIST,
                        new String[]{jobName, jobId},
                        jobName + " 的jobId(" + jobId + ")已存在"
                );
            }
            jobIdSet.add(jobId);
        }
        removeNonexistentJob(stage, jobIdSet);
    }

    private static void removeNonexistentJob(Stage stage, Set<String> jobIdSet) {
        for (Container c : stage.getContainers()) {
            JobControlOption jobControlOption = getJobControlOption(c);
            if (jobControlOption == null) {
                continue;
            }
            boolean isEmpty = jobControlOption.getDependOnId() == null || jobControlOption.getDependOnId().isEmpty();
            if (jobControlOption.getDependOnType() != DependOnType.ID || isEmpty) {
                continue;
            }
            List<String> existJobIds = new ArrayList<>();
            for (String jobId : jobControlOption.getDependOnId()) {
                if (jobIdSet.contains(jobId)) {
                    existJobIds.add(jobId);
                }
            }
            jobControlOption.setDependOnId(existJobIds);
        }
    }

    public static void initDependOn(Stage stage, Map<String, String> params) {
        Map<String, Container> allJobId2JobMap = new HashMap<>();
        for (Container c : stage.getContainers()) {
            String jobId = c.getJobId();
            if (jobId == null || jobId.isEmpty()) {
                continue;
            }
            allJobId2JobMap.put(jobId, c);
        }
        if (allJobId2JobMap.isEmpty()) {
            return;
        }

        Map<String, List<String>> cycleCheckJobMap = new HashMap<>();
        for (Container c : stage.getContainers()) {
            JobControlOption jobControlOption = getJobControlOption(c);
            if (jobControlOption == null) {
                continue;
            }
            List<String> dependOnJobIds = getDependOnJobIds(
                    new DependOnConfig(
                            jobControlOption.getDependOnType(),
                            jobControlOption.getDependOnId(),
                            jobControlOption.getDependOnName()
                    ),
                    params
            );
            if (dependOnJobIds.isEmpty()) {
                continue;
            }
            String jobId = c.getJobId();
            if (jobId != null && !jobId.isEmpty()) {
                cycleCheckJobMap.put(jobId, dependOnJobIds);
            }
            Map<String, String> containerId2JobIds = new HashMap<>();
            for (String dependOnJobId : dependOnJobIds) {
                Container dependOnJob = allJobId2JobMap.get(dependOnJobId);
                if (dependOnJob == null) {
                    continue;
                }
                containerId2JobIds.put(dependOnJob.getId(), dependOnJobId);
            }
            if (!containerId2JobIds.isEmpty()) {
                jobControlOption.setDependOnContainerId2JobIds(containerId2JobIds);
            }
        }

        Map<String, Integer> visited = new HashMap<>();
        for (String jobId : cycleCheckJobMap.keySet()) {
            dsf(jobId, cycleCheckJobMap, visited, stage, allJobId2JobMap);
        }
    }

    public static boolean enableDependOn(Container container) {
        JobControlOption jobControlOption = getJobControlOption(container);
        if (jobControlOption == null) {
            return false;
        }
        switch (jobControlOption.getDependOnType()) {
            case ID:
                return jobControlOption.getDependOnId() != null && !jobControlOption.getDependOnId().isEmpty();
            case NAME:
                return jobControlOption.getDependOnName() != null && !jobControlOption.getDependOnName().isEmpty();
            default:
                return false;
        }
    }

    private static List<String> getDependOnJobIds(DependOnConfig dependOnConfig, Map<String, String> params) {
        switch (dependOnConfig.getDependOnType()) {
            case ID:
                if (dependOnConfig.getDependOnId() == null || dependOnConfig.getDependOnId().isEmpty()) {
                    return new ArrayList<>();
                }
                return dependOnConfig.getDependOnId();
            case NAME:
                if (dependOnConfig.getDependOnName() == null || dependOnConfig.getDependOnName().isEmpty()) {
                    return new ArrayList<>();
                }
                String[] dependONames = regex.split(dependOnConfig.getDependOnName());
                List<String> result = new ArrayList<>();
                for (String name : dependONames) {
                    result.add(EnvUtils.parseEnv(name, params));
                }
                return result;
            default:
                return new ArrayList<>();
        }
    }

    private static boolean dsf(String jobId, Map<String, List<String>> dependOnMap, Map<String, Integer> visited, Stage stage, Map<String, Container> allJobId2JobMap) {
        if (visited.getOrDefault(jobId, 0) == 1) {
            return true;
        }
        if (visited.getOrDefault(jobId, 0) == 2) {
            return false;
        }

        visited.put(jobId, 1);
        List<String> dependOnJobIds = dependOnMap.get(jobId);
        if (dependOnJobIds != null) {
            for (String dependOnJobId : dependOnJobIds) {
                if (dsf(dependOnJobId, dependOnMap, visited, stage, allJobId2JobMap)) {
                    String jobName = getContainerName(stage, allJobId2JobMap.get(jobId), jobId);
                    String dependJobName = getContainerName(stage, allJobId2JobMap.get(dependOnJobId), dependOnJobId);
                    log.error(
                            ProcessMessageCode.ERROR_PIPELINE_DEPENDON_CYCLE,
                            new String[]{jobName, dependJobName}
                    );
                }
            }
        }
        visited.put(jobId, 2);
        return false;
    }

    private static String getContainerName(Stage stage, Container container, String jobId) {
        if (container == null) {
            return jobId;
        }
        String namePrefix = stage.getName();
        if (namePrefix != null && namePrefix.startsWith("stage-")) {
            namePrefix = namePrefix.substring("stage-".length());
        }
        return namePrefix + "-" + container.getId();
    }

    private static JobControlOption getJobControlOption(Container container) {
        if (container instanceof VMBuildContainer) {
            return ((VMBuildContainer) container).getJobControlOption();
        } else if (container instanceof NormalContainer) {
            return ((NormalContainer) container).getJobControlOption();
        }
        return null;
    }
}