package cd.casic.ci.log.scm.service.github;

import cd.casic.ci.common.pipeline.constant.CommonMessageCode;
import cd.casic.ci.log.scm.dal.pojo.github.GithubBranch;
import cd.casic.ci.log.scm.dal.pojo.github.GithubTag;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.util.http.OkhttpUtils;
import cd.casic.framework.commons.util.retry.RetryUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.annotation.Resource;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import okhttp3.Request;
import okhttp3.Response;
import org.springframework.stereotype.Service;

import java.util.Objects;


@Service
@Slf4j
public class GithubService implements IGithubService {

    private static final int HTTP_200 = 200;
    private static final int HTTP_400 = 400;
    private static final int HTTP_401 = 401;
    private static final int HTTP_402 = 402;
    private static final int HTTP_403 = 403;
    private static final int HTTP_404 = 404;
    private static final int HTTP_405 = 405;
    private static final int HTTP_422 = 422;
    private static final int HTTP_500 = 500;

    private static final Long SLEEP_MILLS_FOR_RETRY_500 = 500L;

    private static final String GITHUB_API_URL = "https://api.github.com";

    private static final String OPERATION_GET_BRANCH = "OperationGetBranch";

    private static final String OPERATION_GET_TAG = "OperationGetTag";

    @Resource
    private ObjectMapper objectMapper;

    @Override
    public GithubBranch getBranch(String token, String projectName, String branch) {
        log.info("getBranch| {} - {}", projectName, branch);

        return RetryUtils.execute(new RetryUtils.Action<GithubBranch>() {
            @Override
            public GithubBranch execute() {
                String sBranch = branch != null ? branch : "master";
                String path = "repos/" + projectName + "/branches/" + sBranch;
                Request request = buildGet(token, path);
                String operation = getMessageByLocale(OPERATION_GET_BRANCH, null);
                String body = getBody(operation, request);
                try {
                    return objectMapper.readValue(body, new TypeReference<GithubBranch>() {
                    });
                } catch (JsonProcessingException e) {
                    e.printStackTrace();
                }
                return null;
            }

            @Override
            public GithubBranch fail(Throwable e) throws Throwable {
                log.warn("BKSystemMonitor|getBranch fail| e=" + e.getMessage(), e);
                throw e;
            }

        } , 1, SLEEP_MILLS_FOR_RETRY_500);
    }

    @Override
    public GithubTag getTag(String token, String projectName, String tag) {
        log.info("getTag| {} - {}", projectName, tag);
        return RetryUtils.execute(new RetryUtils.Action<GithubTag>() {
            @Override
            public GithubTag execute() {
                String path = "repos/" + projectName + "/git/refs/tags/" + tag;
                Request request = buildGet(token, path);
                String operation = getMessageByLocale(OPERATION_GET_TAG, null);
                String body = getBody(operation, request);
                try {
                    return objectMapper.readValue(body, new TypeReference<GithubTag>() {
                    });
                } catch (JsonProcessingException e) {
                    e.printStackTrace();
                }
                return null;
            }

            @Override
            public GithubTag fail(Throwable e) throws Throwable {
                log.warn("BKSystemMonitor|getTag fail| e=" + e.getMessage(), e);
                throw e;
            }
        }, 1, SLEEP_MILLS_FOR_RETRY_500);
    }

    private Request buildGet(String token, String path) {
        return request(token, path)
                .get()
                .build();
    }

    private Request.Builder request(String token, String path) {
        return new Request.Builder()
                .url(GITHUB_API_URL + "/" + path)
                .header("Authorization", "token " + token)
                .header("Accept", " application/vnd.github.antiope-preview+json");
    }


    @SneakyThrows
    private String getBody(String operation, Request request) {
        Response response = OkhttpUtils.doHttp(request);
        int code = response.code();
        String message = response.message();
        String body = Objects.requireNonNull(response.body()).string();
        if (log.isDebugEnabled()) {
            log.debug("getBody operation({}). response code({}) message({}) body({})", operation, code, message, body);
        }
        if (!response.isSuccessful()) {
            handException(operation, code);
        }
        return body;
    }

    private void handException(String operation, int code) {
        String msg;
        switch (code) {
            case HTTP_400:
                msg = getMessageByLocale(CommonMessageCode.PARAM_ERROR, null);
                break;
            case HTTP_401:
                msg = getMessageByLocale(CommonMessageCode.AUTH_FAIL, new String[]{"GitHub token"});
                break;
            case HTTP_403:
                msg = getMessageByLocale(CommonMessageCode.ACCOUNT_NO_OPERATION_PERMISSIONS, new String[]{operation});
                break;
            case HTTP_404:
                msg = getMessageByLocale(CommonMessageCode.REPO_NOT_EXIST_OR_NO_OPERATION_PERMISSION, new String[]{"GitHub", operation});
            default:
                msg = "GitHub platform" + operation + "fail";
                break;
        }
        throw new ServiceException(-1000 , msg);
    }

    private String getMessageByLocale(String messageCode, String[] params) {
        return "";
    }
}
